require 'redis'
require 'json'

module Yajq
  class Queue
    def initialize(redis_options = {}, caller_id = $$)
      @redis = Redis.connect(redis_options)
      @caller_id = caller_id
    end

    def add_job job_name, params, &block
      raise QueueException.new("Bad job name") unless job_name.is_a?(String)

      job = {
        'job_id' => Time.now.to_i,
        'caller_id' => @caller_id,
        'name' => job_name,
        'params' => params
      }
      @redis.rpush("yajq:jobs", job.to_json)
      if block_given?
        result = JSON.parse(@redis.blpop("yajq:done:#{@caller_id}", 0)[1])
        block.call result["error"], result["response"]
      end
    end
  end

  class Worker
    def initialize(options = {})
      @redis = Redis.new(options)
      @jobs  = Hash.new
    end

    def start
      while true
        packet =  JSON.parse(@redis.blpop("yajq:jobs", 0)[1])
        puts packet.inspect
        r = Hash.new
        if @jobs.has_key?(packet['name']) then
          begin
            r["response"] = @jobs[packet['name']].call(packet['params'])
          rescue Exception => e
            r["error"] = e
          end
        else
          r["error"] = "unknown command"
        end

        if packet.has_key?("job_id") then
          r["job_id"] = packet["job_id"]
          @redis.rpush("yajq:done:#{packet['caller_id']}", r.to_json)
        end
      end
    end

    def job(name, &block)
      @jobs[name] = block
    end
  end

  class QueueException < Exception; end
end
